function [wcoef, scoef] = FWST_PO(x,L,qmf)

% FWST_PO:  Forward Wavelet Transform (periodized, orthogonal)
% Usage
%              [wcoef, scoef] = FWST_PO(x,L,qmf)
%  Inputs
%    x         1-d signal; length(x) = 2^J.
%    L         Coarsest Level of V_0;  L << J.
%    qmf       Quadrature mirror filter (orthonormal).
%  Outputs
%    wcoef     Wavelet coefficients of x. 
%    scoef     Scaling coefficients of x.

  [n,J] = dyadlength(x) ;
  wcoef = zeros(1,n) ;
  scoef = zeros(1,2*n-1) ;
  
  %take samples at finest scale as beta-coefficients
  beta = ShapeAsRow(x);  
  for j=J-1:-1:L
       alfa = DownDyadHi(beta,qmf);
       wcoef(dyad(j)) = alfa;
       beta = DownDyadLo(beta,qmf) ;  
       scoef(dyad(j)) = beta;
  end
    
  wcoef = wcoef(2^L+1:n);
  wcoef = ShapeLike(wcoef,x);
  wcoef=-wcoef;
  scoef = scoef(2^L+1:n);
  scoef(n-2^L+1:2*n-2^L) = x;
  scoef = ShapeLike(scoef,x);
  
% Acknowledgement
%
% This is modification of the FWT_PO.m routine of WaveLab802 
% in order to have the scaling coefficients as output as well.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
