function s_hat = GBP(s, lambda, distribution, wtrans, wname, NL, max_IP_iter, IP_optimal_tol, ...
    feasible_tol, max_CG_iter, CG_tol, x_save, y_save, z_save, eta_save, rho_save, history_print)

% INPUT
% s = 1D or 2D signal
% lambda = smoothing parameter
% distribution: (currently supported)
%   - 'gaussian'
%   - 'poisson'
% wname = wavelet name, e.g.,
%   - 'haar', 'db4' or 'sym8', etc.
%   file:/usr/local/matlab/help/toolbox/wavelet/wfilters.html?cmdname=wfilters
% NL = nbr of decomposition levels

%warning('!!! FWT2_PO does not work for Gaussian noise if NL>0 and does not work at all for Poisson. The problem is in how the coefficients are stored !!!')

[distribution, s, M, N, signal_dim, PO_Type, PO_Par, PO_qmf] = ...
check_input_variables(s, lambda, distribution, wtrans, wname, NL);

[lambda, x, y, z, eta, inequality_constant, rho, LS, n, nA, n0, length_father] = ...
initial_xyz(s, lambda, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
if(x_save~=0)
  x = x_save; y=y_save; z=z_save; eta=eta_save; rho=rho_save;
end


  [ev, iv, xv, zv] = ...
check_xyz_feasible(s, x, y, z, distribution, wtrans, wname, PO_qmf, NL, signal_dim, inequality_constant, LS, nA, length_father);

% MAIN LOOP

[rx, ry, rz, reta] = KKT_conditions(x, y, z, eta, s, rho, LS, nA, n0, distribution, wtrans, wname, PO_qmf, NL, signal_dim, inequality_constant);
[s_hat, coef_hat, dcf, pcf, primal_feasible, dual_feasible, duality_gap, feasible, converged] = check_convergence(s, x, y, z, eta, rho, feasible_tol, IP_optimal_tol, LS, nA, n0, length_father, distribution, wtrans, wname, PO_qmf, NL, signal_dim, inequality_constant);
continue_iter = (feasible & converged);
hist_scale = length(int2str(max(dcf,pcf)))-1;
dcf=dcf/10^hist_scale;
pcf=pcf/10^hist_scale;
history_iter = [dcf, pcf, primal_feasible, dual_feasible, duality_gap, rho, 0,0,0, ev, iv, xv, zv, 0, 0, 0, feasible, converged];
if(history_print==1) history_iter
end

gamma = 0.95;
IP_iter = 0;
while(~continue_iter)
  %
  % Calculate dx, dy, dz with 1 Newton step to solve approximately ...
  % the non-linear KKT conditions:
  d_diag = x./z;
  r = ry - A_mat(rz ./z-d_diag .* rx, s, LS, nA, n0, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
  p_diag = -der_mu_min(y,s,distribution);
  % 
  h=Newton_right_hand_vector(r, reta, s, d_diag, p_diag, ...
        LS, nA, n0, length_father, distribution, wtrans, wname, ...
        PO_qmf, NL, M, N, signal_dim);  

% For a non-orthonormal transform
  switch wtrans
%    case {'FWT_PO', 'FWT2_PO'}
%      tol = CG_tol/(IP_iter+1);
%      maxit=max_CG_iter;
%      M1=[]; M2=[]; x0=[];
%      restart=10;
%      h=[reshape(r, 1, M*N) (-reta)]';
%      [du,flag,relres,iter,resvec] = ...
%         gmres('linear_operator_bi',h,restart,tol,maxit,M1,M2,x0, ...
%          s, d_diag, p_diag, ...
%          LS, nA, n0, length_father, distribution, wtrans, wname, ...
%          PO_qmf, NL, M, N, signal_dim);
%lou = linear_operator_bi(du, s, d_diag, p_diag, LS, nA, n0, length_father, distribution, wtrans, wname, PO_qmf, NL, M, N, signal_dim, flag);

%  dy=du(1:(M*N))';
%  deta=du((M*N)+(1:(length_father)))';
%  deta=reshape(deta, length_father, 1);
%  dy = reshape(dy, M, N);
%  [Atdy, LS, Phi_0ty] = trans_A_mat(dy, s, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
%  deta=reshape(deta, size(eta,1), size(eta,2));
%  dz = rx - Atdy;
%  dx = (rz - x .* dz)./z;
%  iter=iter(1)*iter(2);
    case {'wavedec', 'wavedec2'}
      tol = (CG_tol/(IP_iter+1)^2);
      maxit=max_CG_iter;
      M1=[]; M2=[]; x0=[];
%     note: there is a negative sign in front of reta to have
%     a symmetric matrix; correspondingly linear_operator_bi is changed.
      h=[reshape(r, 1, M*N) (-reta)]';
      [du,flag,relres,iter,resvec] = ...
      bicg('linear_operator_bi', h, tol,maxit,M1,M2,x0, ...
           s, d_diag, p_diag, ...
           LS, nA, n0, length_father, distribution, wtrans, wname, ...
           PO_qmf, NL, M, N, signal_dim);
      dy=du(1:(M*N))';
      deta=du((M*N)+(1:length_father))';
      dy = reshape(dy, M, N);
      [Atdy, LS, Phi_0ty] = trans_A_mat(dy, s, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
      dz = rx - Atdy;
      dx = (rz - x .* dz)./z;
    case {'Qwavedec', 'Qwavedec2'}
      tol = (CG_tol/10/(IP_iter+1))^2;
      maxit=2*max_CG_iter;
      M1=[]; M2=[]; x0=[];
      [du,flag,relres,iter,resvec] = ...
      pcg('linear_operator_squared', h, tol,maxit,M1,M2,x0, ...
          s, d_diag, p_diag, ...
          LS, nA, n0, length_father, distribution, wtrans, wname, ...
          PO_qmf, NL, M, N, signal_dim);
      dy=du(1:(M*N))';
      deta=du((M*N)+(1:length_father))';
      dy = reshape(dy, M, N);
      [Atdy, LS, Phi_0ty] = trans_A_mat(dy, s, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
      dz = rx - Atdy;
      dx = (rz - x .* dz)./z;
%   For an orthonormal transform
    case {'FWT_PO', 'FWT2_PO'}
      tol = CG_tol/(IP_iter+1);
      maxit=max_CG_iter;
      M1=[]; M2=[]; x0=[];
      [du,flag,relres,iter,resvec] = ...
        pcg('linear_operator_ortho', h, tol,maxit,M1,M2,x0, ...
            s, d_diag, p_diag, ...
            LS, nA, n0, length_father, distribution, wtrans, wname, ...
            PO_qmf, NL, M, N, signal_dim);
      dy = synthesis([reta du'], LS, wtrans, wname, PO_qmf, NL, signal_dim);
      dy = reshape(dy, M, N);
      [Aty, LS, Phi_0ty] = trans_A_mat(dy, s, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
      dz = rx - Aty;
      dx = (rz - x .* dz)./z;
      dy = reshape(dy, 1, M*N);
      u=reshape(p_diag .* dy, M, N) - r;
      switch distribution
        case 'poisson'
          if(n0>0)
            d0 = d_diag(1:n0);
            dy0 = dy(s==0);
            d_diag_dy0 = d0 .* dy0;
          else
            d_diag_dy0=[];
          end
          nzeros = zeros(M, N); 
          nzeros(s==0) = d_diag_dy0;
          u = u + nzeros;
      end 
      [Aty, LS, deta] = trans_A_mat(u, s, distribution, wtrans, wname, ...
                                    PO_qmf, NL, signal_dim);
      dy = reshape(dy, M, N);
  end
  % Find longest Newton step while remaining feasible:
  [beta_x, beta_y, beta_z, beta_eta] = ...
    length_Newton_step(s, x, y, z, eta, dx, dy, dz, deta, ...
                       wtrans, wname, PO_qmf, NL, ...
                       signal_dim, inequality_constant, LS, nA, n0, ...
  distribution, length_father);
  % Newton step
  x = x + gamma * beta_x * dx;
  y = y + gamma * beta_y * dy;
  z = z + gamma * beta_z * dz;
  eta = eta + gamma * beta_eta * deta;
  rho = (1 - min([beta_x, beta_y, beta_z, gamma])) * rho;
  % update, checkup
  [rx, ry, rz, reta] = KKT_conditions(x, y, z, eta, s, rho, LS, nA, n0, distribution, wtrans, wname, PO_qmf, NL, signal_dim, inequality_constant);
  [s_hat, coef_hat, dcf, pcf, primal_feasible, dual_feasible, duality_gap, feasible, converged] = check_convergence(s, x, y, z, eta, rho, feasible_tol, IP_optimal_tol, LS, nA, n0, length_father, distribution, wtrans, wname, PO_qmf, NL, signal_dim, inequality_constant);
  [ev, iv, xv, zv] = ...
  check_xyz_feasible(s, x, y, z, distribution, wtrans, wname, PO_qmf, NL, ...
                     signal_dim, inequality_constant, LS, nA, length_father);
  dcf=dcf/10^hist_scale;
  pcf=pcf/10^hist_scale;
  history_new = [dcf, pcf, primal_feasible, dual_feasible, duality_gap, rho, beta_x, beta_y, beta_z, ev, iv, xv, zv, flag,relres,iter, feasible, converged];
  history_iter = [history_iter'  history_new']';
  if(history_print==1) history_new
  end
  IP_iter = IP_iter+1;
  continue_iter = ((feasible & converged) | (IP_iter > max_IP_iter));
% continue_iter = ((feasible | converged) | (IP_iter > max_IP_iter));
%  if(any([iv, xv, zv])) continue_iter = 1; end
end
if(history_print==2) history_iter
end


