function f = MakeIntensities(name,n)

% MakeIntensities:     Make artificial 1-d Poisson intensities.
% Usage
% 	                    f = MakeIntensities(name,n)
% Inputs 
%  name                String: 
%                      'Smooth','Clipped Blocks','Bumps','Angles','Spikes','Bursts'.
% 	n 			           Desired signal length.
% Ouputs
%	f		   	        Artificial 1-d Poisson intensities.

%Making grid points
t = linspace(0,1,n);

if strcmp(name,'Smooth')
   sig = 4*(6.75*t.^2.*(1-t)).^3;
elseif strcmp(name,'Clipped Blocks')
	    pos = [ .1 .13 .15 .23 .25 .40 .44 .65  .76 .78 .81];
	    hgt = [4 (-5) 3 (-4) 5 (-4.2) 2.1 4.3  (-3.1) 2.1 (-4.2)];
	    sig = zeros(size(t));
	    for j=1:length(pos)
	        sig = sig + (1 + sign(t-pos(j))).*(hgt(j)/2) ;
       end
       sig(sig < 0)=0;
elseif strcmp(name,'Bumps')
       pos = [ .1 .13 .15 .23 .25 .40 .44 .65  .76 .78 .81];
    	 hgt = [ 4  5   3   4  5  4.2 2.1 4.3  3.1 5.1 4.2];
    	 wth = [.005 .005 .006 .01 .01 .03 .01 .01  .005 .008 .005];
    	 sig = zeros(size(t));
    	 for j =1:length(pos)
       	  sig = sig + hgt(j)./(( 1 + (abs(t - pos(j))./wth(j))).^4);
       end 
elseif strcmp(name,'Angles')
       sig = ((2*t + 0.5).*(t <= 0.15)) + ...
       ((-12*(t-0.15) + 0.8).*(t > 0.15 & t <= 0.2)) + ...
       0.2*(t > 0.2 & t <= 0.5) + ...
       ((6*(t - 0.5) + 0.2).*(t > 0.5 & t <= 0.6)) + ...
       ((-10*(t - 0.6) + 0.8).*(t > 0.6 & t <= 0.65)) + ...
       ((-0.5*(t - 0.65) + 0.3).*(t > 0.65 & t <= 0.85)) + ...
       ((2*(t - 0.85) + 0.2).*(t > 0.85));
       sig = (5./range(sig)).*sig - 1.6;
elseif strcmp(name,'Spikes')
		 sig = 15.6676 .* (exp(-500.*(t-0.23).^2) + ...
		 2.*exp(-2000.*(t-0.33).^2) +  4.*exp(-8000.*(t-0.47).^2) + ...
		 3.*exp(-16000.*(t-0.69).^2) +exp(-32000.*(t-0.83).^2) );
		 sig=(5./range(sig)).*sig;
elseif strcmp(name,'Bursts') 
       I_1 = exp(-(abs(t-0.2)./0.01).^1.2).*(t<=0.2) + exp(-(abs(t-0.2)./0.03).^1.2).*(t>0.2);
       I_2 = exp(-(abs(t-0.3)./0.01).^1.2).*(t<=0.3) + exp(-(abs(t-0.3)./0.03).^1.2).*(t>0.3);
       I_3 = exp(-(abs(t-0.4)./0.01).^1.2).*(t<=0.4) + exp(-(abs(t-0.4)./0.03).^1.2).*(t>0.4);
       sig = (4.*I_1+3.*I_2+4.5.*I_3);
else
 	   disp('MakeSignalPo: I don*t recognize'); name
       disp('Allowable Names are:')
       disp('Smooth'),
       disp('Clipped Blocks'),
       disp('Bumps'),
       disp('Angles'),
       disp('Spikes'),
       disp('Bursts')
end

%Creating artificial 1-d signal 
f = sig;

% Acknowledgement
%
% This code is based on a MatLab routine kindly provided by Jonathan Buckheit, Shaobing Chen, David Donoho,
% Iain Johnstone and Jeffrey Scargle.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
 