function f = MakeNoisyIntensities(name,n,scale)

% MakeNoisyIntensities:   Generate Poisson data from a specified function
%                         with various count settings.
% Usage
%                         f = MakeNoisyIntensities(name,n,scale)
% Inputs
%  name                   String: 
%                         'Smooth','Clipped Blocks','Bumps','Angles','Spikes','Bursts'.
% 	n 			              Desired signal length.
%  scale                  Scaling factor for constructing 'low', 'medium' and 
%                         'high' count settings.
% Outputs
%   f		                 Artificial 1-d Poisson data.

%Artificial intensity
fval=MakeIntensities(name,n);

%Scaled artificial intensity
scalfval = scale * (1 + fval);

%Artificial Poisson time series data
f=poissrnd(scalfval)+eps; 

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
