function spprob = TISumProd(lprmat)

% TISumPro.m:  Compute sums of products of probabilities arising
%              from TI model in BMSM calculations.
% Usage        
%              spprob = TISumProd(lprmat)
% Input
% lprmat       Matrix (Jxn) of (log) probabilities.
% Output
% spprob       Sum of products of probabilities.

[J,n] = size(lprmat);
spprob = exp( lprmat(J,[1:2:n-1]) ) + exp( lprmat(J,[2:2:n]) );
for j = J-1:-1:1,
  nrow = 2^(J-j);  ncol = 2^j;
  tempprob = sum( reshape(lprmat(j,:),nrow,ncol) );
  spprob = sum( [tempprob; log(spprob)] );
  spprob = exp(spprob);
  spprob = sum(  reshape(spprob,2,ncol/2)  );
end

% This is what the algorithm would look like without using probabilities on the log scale.
% prmat = lprmat;
% spprob = prmat(J,[1:2:n-1]) + prmat(J,[2:2:n]);
% for j = J-1:-1:1,
%     nrow = 2^(J-j); ncol = 2^j;  
%     spprob = spprob.*(  prod(reshape(prmat(j,:),nrow,ncol))  );
%     spprob = sum(  reshape(spprob,2,ncol/2)  );
% end

% Copyright (c) 1998
%
% Eric Kolaczyk, 
% Department of Mathematics and Statistics
% Boston University
% 111 Cummington Avenue
% Boston, MA 02215
% USA
%
% mailto: kolaczyk@math.bu.edu 
%
% Revised: 01/09/05


