% Load the original 1-D signal
load 'burst551.mat'
photon_times=burst3;

% Binning of the original data 
num_bins=1024;
t_min = min(photon_times);
t_max = max(photon_times);
t_range = t_max - t_min;
dt_bin = t_range / num_bins;
mid_first = t_min + dt_bin/2;
mid_last  = t_max - dt_bin/2;
data_bins = linspace(mid_first,mid_last,num_bins);
poissig = hist(photon_times,data_bins);
x=data_bins';
 
% Intensity function estimates
 
% Translation invariant version of Anscombe-based estimate with hard minimax threshold     
j0=6; % Primary resolution level
h=MakeONFilter('Symmlet',8); % Wavelet filter choice
anscombeMin_TI_H = recanscombeTI(poissig,'MinMax','Hard',j0,h);

% Translation invariant version of Haar estimate with hard threshold
% Estimate of background intensity level
mu0=ceil(mean(poissig(floor(2/5*num_bins)+1:num_bins))); 
j0=4; % Primary resolution level
haar_TI_H = rechaarTI(poissig,mu0,'Hard',j0);

% Translation invariant version of Bayesian multiscale model estimate      
pqind=2;  % Data-adaptive hyperparameters
[BMSMShrink_TI,ppp,qqq] = BMSMShrink(poissig, pqind);

% Plotting the results
% Changing the values of the xaxis -- seconds
xsec = x./10^6;
n2=num_bins/2;

set(gcf,'papertype','A4');
set(gcf,'paperposition',[1, 2.5, 6.5, 8.00]);
subplot(4,1,1)
plot(xsec,poissig,'b')
axis([0 xsec(n2) 0 50])
title('BATSE trigger 551 - energy channel 3','FontSize',8)

subplot(4,1,2)
plot(xsec,anscombeMin_TI_H,'g')
axis([0 xsec(n2) 0 50])
title('ANSCOMBE-MIN-TI-H','FontSize',8)

subplot(4,1,3)
plot(xsec,haar_TI_H,'g')
axis([0 xsec(n2) 0 50])
title('HAAR-TI-H','FontSize',8)

subplot(4,1,4)
plot(xsec,BMSMShrink_TI,'g')
axis([0 xsec(n2) 0 50])
title('BMSM-TI','FontSize',8)

print -deps exampleBATSE.eps

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone"
% Sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 