function f = fiszinv(signal)

% fiszinv:   The inverse Fisz transformation applied on the reconstructed
%            signal after using Gaussian-based thresholds.
% Usage
%            f = fiszinv(signal)
% Inputs
%   signal	 1-d reconstructed signal, length(signal)= 2^J.
% Outputs
%   f        The estimate of the Poisson intensity function.
% References
%            Fisz, M. (1955). The limiting distribution of a function of two
%            independent random variables and its statistical applications.
%            "Colloquium Mathematicum", 3, 138-146.
%
%            Fryzlewick, P. & Nason, G.P. (2001). Poisson intensity estimation
%            using wavelets and the Fisz transformation. "Technical Report", 01-10,
%            Department of Mathematics, University of Bristol, UK.

a = 2;
n = length(signal);
nhalf = n/2;
J=log2(n);
res = signal;
sm = zeros(1, nhalf);
dett = sm;
for i=1:J
   sm(1:nhalf) = (res(2*(1:nhalf)-1) + res(2*(1:nhalf)))/a;
   dett(1:nhalf) = (res(2*(1:nhalf)-1) - res(2*(1:nhalf)))/a;
   res(1:nhalf) = sm(1:nhalf);
   res((nhalf+1):n) = dett(1:nhalf);
   n = n/2;
   nhalf = nhalf/2;
end
nhalf = 1;
n = 2;
for i = 1:J
   sm(1:nhalf) = res(1:nhalf);
   dett(1:nhalf) = res((nhalf + 1):n);
   res(2*(1:nhalf)-1) = a/2 * (sm(1:nhalf) + dett(1:nhalf).*sqrt(sm(1:nhalf)));
   res(2*(1:nhalf)) = a/2 * (sm(1:nhalf) - dett(1:nhalf).*sqrt(sm(1:nhalf)));
   res(res(1:n) < 0) = 0;
   n = 2*n;
   nhalf = 2*nhalf;
end
f = res;

% Acknowledgement
%
% This code is based on an Splus routine kindly provided by Piotr Fryzlewicz.
%
% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone" 
% sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
