function [lambda, x, y, z, eta, inequality_constant, rho, LS, n, nA, n0, length_father] = ...
initial_xyz(s, lambda, distribution, wtrans, wname, PO_qmf, NL, signal_dim)
% x=Primal
% y=Dual
% z=Slack
% eta=Lagrange multiplier for the equality constraints
%   (x,y,z,eta) must be feasible
% rho=log-barrier weight
% n=nbr of observations
% nA=nbr of wavelet (approx.+detail) coefficients
% n0=nbr of observations==0 for the Poisson case
% length_father = p0 = nbr of approximation coefficients.

switch distribution
  case 'gaussian'
    [x, LS] = analysis(s, wtrans, wname, PO_qmf, NL, signal_dim);
  case 'poisson'
    [x, LS] = analysis(s, wtrans, wname, PO_qmf, NL, signal_dim);
end
if(signal_dim==1)
  length_father = LS(1);
  n = LS(end);
  nA = length(x);
else
  length_father = LS(1,1)*LS(1,2);
  N=LS(end,1); M=LS(end,2);
  n = N*M;
  nA = length(x);
end
% set lambda to its universal value if lambda=0
if(lambda==0)
  switch distribution
    case 'gaussian'
      lambda = sqrt(2*log(n));
      warning('sigma not estimated yet!!!')
    case 'poisson'
      mu0 = estimate_mu0(s, wtrans, wname, PO_qmf, NL, signal_dim);
      [x, LS] = analysis(mu0, wtrans, wname, PO_qmf, NL, signal_dim);
      x(x==0)=.1;
      length_support = size(PO_qmf,2);
      Cst=max(power(PO_qmf*sqrt(length_support),2));
      M2=Cst*trimmean(reshape(1./mu0, 1, size(mu0,1)*size(mu0,2)),10);
      lambda=sqrt(M2*2*log(n)/n);
  end
end
eta=-x(1:length_father);
x_mother=x((length_father+1):nA);
x_pos = x_mother .* (x_mother>0);
x_neg = -x_mother .* (x_mother<0);
switch distribution
  case 'gaussian'
    % constants
    n0=0;
    lambda_ones = lambda * ones(1, nA - length_father);
    inequality_constant=[lambda_ones, lambda_ones];
    % x y z eta
    x = [x_pos, x_neg];
    y = A_mat(x, s, LS, nA, n0, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
    omega = analysis(y, wtrans, wname, PO_qmf, NL, signal_dim);
    omega_bar = 1.1 * max(abs(omega))/min(lambda_ones);
    y = y/omega_bar;
    omega = omega/omega_bar;
    z1 = lambda_ones - omega((length_father+1):nA);
    z2 = lambda_ones + omega((length_father+1):nA);
    z = [z1, z2];
    x = x+.1;
    z = z+.1;
    % log-barrier penalty
    [rx, ry, rz, reta] = KKT_conditions(x, y, z, eta, s, 0, LS, nA, n0, ...
                                        distribution, wtrans, wname, ...
                                        PO_qmf, NL, ...
                                        signal_dim, inequality_constant);
    ry = reshape(ry, 1, size(ry, 1)*size(ry, 2));
    rho = (0.01 * norm([rx ry rz reta]))/sqrt(2*nA);
  case 'poisson'
    % constants
    weights = [];
    if(strcmp(wtrans, 'FWT_PO'))
      for(i = power(2, NL:(log2(n)-1)))
        weights_i=ones(1, i)*sqrt(i);
        weights = [weights weights_i];
      end
    end
    if(strcmp(wtrans, 'FWT2_PO'))
      NLM=NL; NLN=NL;
      weight_matrix = zeros(LS(2,1), LS(2,2));
      indexM=[1 (power(2,NLM:log2(M/2))+1)];
      lengthM=[2^NLM power(2,NLM:log2(M/2))];
      indexN=[1 (power(2,NLN:log2(N/2))+1)];
      lengthN=[2^NLN power(2,NLN:log2(N/2))];
      for(i = 2:size(indexM,2))
        weight_matrix(indexM(i):(indexM(i)+lengthM(i)-1),...
                      1:(lengthN(i)))=sqrt(lengthM(i))*sqrt(lengthN(i));
        weight_matrix(indexM(i):(indexM(i)+lengthM(i)-1),...
                      (lengthN(i)+1):(2*lengthN(i)))=...
                      sqrt(lengthM(i))*sqrt(lengthN(i));
        weight_matrix(1:(lengthM(i)),...
                      (lengthN(i)+1):(2*lengthN(i)))=...
                      sqrt(lengthM(i))*sqrt(lengthN(i));
      end
      weight_matrix(1:power(2, NLN), 1:power(2, NLM))=Inf;
      weights=weight_matrix(weight_matrix~=Inf)';
    end
    if(strcmp(wtrans, 'wavedec'))
      LS1end=LS; LS1end(1)=[]; LS1end(end)=[];
      for(i = LS1end)
        weights_i=ones(1, i)*sqrt(i);
        weights = [weights weights_i];
      end
    end
    if(strcmp(wtrans, 'wavedec2'))
      for(i=[2:(size(LS,1)-1)])
        weights_i=ones(1, 3*LS(i,1)*LS(i,2))*sqrt(LS(i,1))*sqrt(LS(i,2));
        weights = [weights weights_i];
      end
    end
    lambda_weights = lambda*weights;
    n0 = sum(sum(s==0));
    inequality_constant=[ones(1, n0), lambda_weights, lambda_weights]; 
    % x y z eta
    x = [zeros(1, n0), x_pos, x_neg];
    y = A_mat(x, s, LS, nA, n0, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
    omega = trans_A_mat(y, s, distribution, wtrans, wname, PO_qmf, NL, signal_dim);
    omega_bar = 1.1 * max([abs(omega) abs(y)])/min([lambda_weights 1]);
    y = y/omega_bar;
    omega = omega/omega_bar;
    z0 = (reshape(y(s==0), 1, n0)+1);
    z1 = lambda_weights - omega(n0+(1:(nA-length_father)));
    z2 = lambda_weights + omega(n0+(1:(nA-length_father)));
    z = [z0, z1, z2];
    x = x+.1;
    z = z+.1;
    % log-barrier penalty
    [rx, ry, rz, reta] = KKT_conditions(x, y, z, eta, s, 0, LS, nA, n0, ...
                                        distribution, ...
                                        wtrans, wname, PO_qmf, NL, ...
                                        signal_dim, inequality_constant);
    ry = reshape(ry, 1, size(ry, 1)*size(ry, 2));
    rho = (0.01 * norm([rx ry rz reta]))/sqrt(2*nA);
end












