% Variables that user has to define
% 
% test:     defines the burst; test = 1:6.
% channel:  defines the channel used to measure; channel = 1:4.
% num_bins: defines the number of bins; num_bins=256, 512, or 1024.
% method:   defines the wavelet estimator; method = 1:37.
% lev:      defines the coarsest level of resolution.
% h:        defines the quadrature mirror filter. 
% ordind:   defines the order indicator dictating whether skewness alone is corrected for, 
%           or both skewness and kurtosis are corrected for.
%           Enter 3 if you want skewness only; 
%           Enter 4 if you want both skweness and kurtosis.
% pqind:    defines the method used to elicitate the hyperparameters in BMSM-TI estimator.
%           Enter 1 if you use the fixed values.
%           Enter 2 if you use the EM algorithm.
% wname:    defines the wavelet filter to be used in GBP estimator.
%           Enter 'ha' if you want the haar filter.
%           Enter 'db' if you want the daubechies 4 filter.
%           Enter 'co' if you want the coiflet 3 filter.
%           Enter 'sy' if you want the symmlet 4 filter.
%***********************************************************************************************************************
% Wavelet methods used : 
%                  Anscombe Transformation: 1)  hard minimax threshold
%                                           2)  soft minimax threshold
%                                           3)  hard universal threshold
%                                           4)  soft universal threshold
%                                           5)  hard cross validation threshold
%                                           6)  soft cross validation threshold
%                                           7)  translation invariant hard minimax threshold
%                                           8)  translation invariant soft minimax threshold
%                                           9)  translation invariant hard universal threshold
%                                           10) translation invariant soft universal threshold
%                                           11) translation invariant hard cross validation threshold
%                                           12) translation invariant soft cross validation threshold
%                  Fisz Transformation:     13) hard minimax threshold
%                                           14) soft minimax threshold
%                                           15) hard universal threshold
%                                           16) soft universal threshold
%                                           17) hard cross validation threshold
%                                           18) soft cross validation threshold
%                                           19) translation invariant hard minimax threshold
%                                           20) translation invariant soft minimax threshold
%                                           21) translation invariant hard universal threshold
%                                           22) translation invariant soft universal threshold
%                                           23) translation invariant hard cross validation threshold
%                                           24) translation invariant soft cross validation threshold
%                  Haar Thresholds:         25) translation invariant hard threshold
%                                           26) translation invariant soft threshold
%                  Corrected Thresholds:    27) hard threshold
%                                           28) soft threshold
%                                           29) translation invariant hard threshold
%                                           30) translation invariant soft threshold
%                                           
%                  31) Modulation Estimators
%                  32) Translation Invariant Modulation Estimators
%                  33) Multmodulation Estimators
%                  34) Translation Invariant Multmodulation Estimators
%                  35) Translation Invariant Invariant Bayesian Multiscale Method 
%                  36) Translation Invariant Invariant Multiscale Multiplicative Innovations Model 
%                  37) l1-Penalised Likelihood Method
%******************************************************************************************************************
% Notes:
% if method=1:24  it is necessary to define lev and h
% if method=25:26 it is necessary to define lev
% if method=27:30 it is necessary to define lev, h, and ordind
% if method=31:34 it is necessary to define lev and h
% if method=35 it is necessary to define pqind
% if method=36 it is necessary to define lev
% if method=37 it is necessary to define wname
%******************************************************************************************************************

test=5;
channel=3;
method=1;
num_bins=256;
if test==1
    load 'burst138.mat'
    nburst=138;
    if channel==1
       photon_times=burst138ch1;
    elseif channel==2
       photon_times=burst138ch2;
    elseif channel==3
       photon_times=burst138ch3;
    elseif channel==4
       photon_times=burst138ch4;
    end
elseif test==2
    load 'burst185.mat'
    nburst=185;
    if channel==1
       photon_times=burst185ch1;
    elseif channel==2
       photon_times=burst185ch2;
    elseif channel==3
       photon_times=burst185ch3;
    elseif channel==4
       photon_times=burst185ch4;
    end
elseif test==3
    load 'burst207.mat'
    nburst=207;
    if channel==1
       photon_times=burst207ch1;
    elseif channel==2
       photon_times=burst207ch2;
    elseif channel==3
       photon_times=burst207ch3;
    elseif channel==4
       photon_times=burst207ch4;
    end
elseif test==4
    load 'burst474.mat'
    nburst=474;
    if channel==1
       photon_times=burst474ch1;
    elseif channel==2
       photon_times=burst474ch2;
    elseif channel==3
       photon_times=burst474ch3;
    elseif channel==4
       photon_times=burst474ch4;
    end
elseif test==5
    load 'burst551.mat'
    nburst=551;
    if channel==1
       photon_times=burst1;
    elseif channel==2
       photon_times=burst2;
    elseif channel==3
       photon_times=burst3;
    elseif channel==4
       photon_times=burst4;
    end
elseif test==6
    load 'burst845.mat'
    nburst=845;
    if channel==1
       photon_times=burst845ch1;
    elseif channel==2
       photon_times=burst845ch2;
    elseif channel==3
       photon_times=burst845ch3;
    elseif channel==4
       photon_times=burst845ch4;
    end
end
%
if 1 <= method & method <= 24
   lev=6;
   h=MakeONFilter('Symmlet',8);
   f = binning(test,channel,nburst,photon_times,num_bins,method,lev,h,[],[],[]);
end
if method == 25 | method == 26
   lev=4;
   f = binning(test,channel,nburst,photon_times,num_bins,method,lev,[],[],[],[]);
end
if 27 <= method & method <= 30
   lev=4;
   h=MakeONFilter('Symmlet',8);
   ordind=4;
   f = binning(test,channel,nburst,photon_times,num_bins,method,lev,h,ordind,[],[]);
end
if 31 <= method & method <= 34
   lev=4;
   h=MakeONFilter('Symmlet',8);
   f = binning(test,channel,nburst,photon_times,num_bins,method,lev,h,[],[],[]);
end
if method == 35
   pqind=1;
   f = binning(test,channel,nburst,photon_times,num_bins,method,[],[],[],pqind,[]);
end
if method == 36
   lev=4;
   f = binning(test,channel,nburst,photon_times,num_bins,method,lev,[],[],[],[]);
end
if method == 37
   wname='db';
   f = binning(test,channel,nburst,photon_times,num_bins,method,[],[],[],[],wname);
end
clear all;

% Copyright (c) 2002
%
% Panagiotis Besbeas
% Institute of Mathematics and Statistics
% University of Kent at Canterbury
% Canterbury, Kent CT2 7NF
% United Kingdom
%
% mailto: P.T.Besbeas@ukc.ac.uk
%
% and
%
% Italia De Feis
% Istituto per le Applicazioni del Calcolo "Mauro Picone"
% Sezione di Napoli - CNR
% Via Pietro Castellino 111
% 80131 Napoli
% Italy
%
% mailto: defeis@iam.na.cnr.it
%
% and
%
% Theofanis Sapatinas
% Department of Mathematics and Statistics
% University of Cyprus
% P.O. Box 20537  
% CY 1678 Nicosia
% Cyprus
%
% mailto: T.Sapatinas@ucy.ac.cy 
 

