function ss = sfunc(p,q,x0,x1)

% sfunc.m: Code to compute shrinkage function for BMSM estimation.
%          This function is just a (scalar) posterior mean.
% Usage    
%          ss = sfunc(p,q,x0,x1) 
%Inputs
%  p       Mixing probability.
%  q       Beta shape parameter.
%  x0      Vector of left x's.
%  x1      Vector of right x's.
% Output
%  ss      Shrinkage factor in moving from scale j to j+1.

xx = x0 + x1;
if (p == 1),
    ss = 0.5*ones(1,length(x0));
elseif (p == 0),
    ss = (x0 + q)./(xx + 2*q);
 else
    
% Compute first half denomenator of sum.
  fhd = log(1-p) - log(p) + (xx+1).*log(2) + ...
        betaln(x0+q,x1+q) - betaln(q,q);
     fhd = 2 + exp(fhd);
     
% Compute second half denomenator.
  shd1 = log(p) - log(1-p) + betaln(q,q) - ...
        xx.*log(2) - betaln(x0+q+1,x1+q);
     shd  = exp(shd1) + (xx + 2.*q)./(x0 + q);
     
% Put together two pieces.  Numerators are just 1.
  ss = (1./fhd) + (1./shd);
end

% Copyright (c) 1998
%
% Eric Kolaczyk, 
% Department of Mathematics and Statistics
% Boston University
% 111 Cummington Avenue
% Boston, MA 02215
% USA
%
% mailto: kolaczyk@math.bu.edu 
%
% Revised: 01/09/05


